@@ -1,6 +1,6 @@
 /*
 	TiMidity++ -- MIDI to WAVE converter and player
-	Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+	Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
 	Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
 	This program is free software; you can redistribute it and/or modify
@@ -82,14 +82,23 @@ static BOOL APIENTRY PrefTiMidity4DialogProc(HWND hwnd, UINT uMess, WPARAM wPara
 static int DlgOpenConfigFile(char *Filename, HWND hwnd);
 static int DlgOpenOutputFile(char *Filename, HWND hwnd);
 
-#if defined(__CYGWIN32__) || defined(__MINGW32__)
+//#if defined(__CYGWIN32__) || defined(__MINGW32__)
+#if 0 /* New version of mingw */
 //#define pszTemplate	u1.pszTemplate
 //#define pszIcon		u2.pszIcon
+#ifndef NONAMELESSUNION
 #define NONAMELESSUNION
+#endif
+#ifndef DUMMYUNIONNAME
 #define DUMMYUNIONNAME	u1
+#endif
+#ifndef DUMMYUNIONNAME2
 #define DUMMYUNIONNAME2	u2
+#endif
+#ifndef DUMMYUNIONNAME3
 #define DUMMYUNIONNAME3	u3
 #endif
+#endif
 
 #define PREFWND_NPAGES 5
 void PrefWndCreate(HWND hwnd)
@@ -402,7 +411,7 @@ PrefPlayerDialogProc(HWND hwnd, UINT uMess, WPARAM wParam, LPARAM lParam)
 			break;
 		case PSN_APPLY:
 			PrefSettingApply();
-			PropSheet_UnChanged(hPrefWnd,hwnd);
+			PropSheet_UnChanged((HWND)hPrefWnd,hwnd);
 			break;
 		case PSN_SETACTIVE:
 			SetDlgItemText(hwnd,IDC_EDIT_CONFIG_FILE,TEXT(sp_temp->ConfigFile));
