@@ -1,6 +1,6 @@
 /*
-    TiMidity -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    TiMidity++ -- MIDI to WAVE converter and player
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <toivonen@clinet.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -64,12 +64,12 @@ extern int opt_evil_mode;
 extern int opt_buffer_fragments;
 extern int32 opt_output_rate;
 extern int PlayerLanguage;
-extern volatile int data_block_time;	// msec
+extern volatile int data_block_bits;
 extern volatile int data_block_num;
 extern int DocWndIndependent;
 extern int SeachDirRecursive;
 extern int IniFileAutoSave;
-
+extern int SecondMode;
 
 //*****************************************************************************/
 // ini
@@ -316,6 +316,7 @@ ApplySettingPlayer(SETTING_PLAYER *sp)
   DocWndIndependent = sp->DocWndIndependent; 
   SeachDirRecursive = sp->SeachDirRecursive;
   IniFileAutoSave = sp->IniFileAutoSave;
+  SecondMode = sp->SecondMode;
 }
 
 void
@@ -375,6 +376,7 @@ SaveSettingPlayer(SETTING_PLAYER *sp)
   sp->DocWndIndependent = DocWndIndependent; 
   sp->SeachDirRecursive = SeachDirRecursive;
   sp->IniFileAutoSave = IniFileAutoSave;
+  sp->SecondMode = SecondMode;
 }
 
 extern int set_play_mode(char *cp);
@@ -477,7 +479,7 @@ ApplySettingTiMidity(SETTING_TIMIDITY *st)
 	control_ratio = play_mode->rate / CONTROLS_PER_SECOND;
 	control_ratio = SetValue(control_ratio, 1, MAX_CONTROL_RATIO);
     }
-    data_block_time = st->data_block_time;
+    data_block_bits = st->data_block_bits;
     data_block_num = st->data_block_num;
 }
 
@@ -589,7 +591,7 @@ SaveSettingTiMidity(SETTING_TIMIDITY *st)
 #if defined(__W32__) && defined(SMFCONV)
     st->opt_rcpcv_dll = opt_rcpcv_dll;
 #endif /* SMFCONV */
-  st->data_block_time = data_block_time;
+  st->data_block_bits = data_block_bits;
   st->data_block_num = data_block_num;
 }
 
