@@ -1,6 +1,6 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -1956,6 +1956,7 @@ void x_sry_wrdt_apply(uint8 *data, int len)
       case 0x1d:
       case 0x1e:
       case 0x1f:
+      case 0x20:
       case 0x71:
       case 0x72:
       case 0x7f:
@@ -2052,10 +2053,6 @@ static ImagePixmap *create_shm_image_pixmap(int width, int height, int depth)
 	goto done;
     }
 
-    /* allocate n-depth Z image data structure */
-    ip->im->data = (char *)safe_large_malloc(ip->im->bytes_per_line *
-					     ip->im->height);
-
     /* The next step is to create the shared memory segment.
      * The return value of shmat() should be stored both
      * the XImage structure and the shminfo structure.
