@@ -1,6 +1,6 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -408,7 +408,7 @@ static int ctl_blocking_read(int32 *valp) {
         opt_modulation_wheel = n & MODUL_BIT;
         opt_portamento = n & PORTA_BIT;
         opt_nrpn_vibrato = n & NRPNV_BIT;
-        opt_reverb_control = n & REVERB_BIT;
+        opt_reverb_control = !!(n & REVERB_BIT);
         opt_channel_pressure = n & CHPRESSURE_BIT;
         opt_overlap_voice_allow = n & OVERLAPV_BIT;
         opt_trace_text_meta_event = n & TXTMETA_BIT;
