@@ -1,6 +1,6 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -459,10 +459,19 @@ void *safe_malloc(size_t count)
 		  "Strange, I feel like allocating %d bytes. "
 		  "This must be a bug.", count);
     }
+#ifndef	ORIGINAL
+    else if(count == 0) {
+	/* To be safe: don't ever try to allocate 0 bytes
+	 * Different malloc's behave differently.
+	 */
+	return safe_malloc(1);
+    }
+#endif	/* Not ORIGINAL (RAKK/HIOENS) */
     else if((p = (void *)malloc(count)) != NULL)
 	return p;
     else
     {
+#ifdef	ORIGINAL
 	if(count == 0) {
 	    /* Some malloc routine return NULL if count is zero, such as
 	     * malloc routine from libmalloc.a of Solaris.
@@ -470,6 +479,7 @@ void *safe_malloc(size_t count)
 	     */
 	    return safe_malloc(1);
 	}
+#endif	/* ORIGINAL */
 
 	errflag = 1;
 	ctl->cmsg(CMSG_FATAL, VERB_NORMAL,
@@ -489,6 +499,14 @@ void *safe_large_malloc(size_t count)
 
     if(errflag)
 	safe_exit(10);
+#ifndef	ORIGINAL
+    if(count == 0) {
+	/* To be safe: don't ever try to allocate 0 bytes
+	 * Different malloc's behave differently.
+	 */
+	return safe_large_malloc(1);
+    } else
+#endif	/* Not ORIGINAL (RAKK/HIOENS) */
     if((p = (void *)malloc(count)) != NULL)
 	return p;
     errflag = 1;
@@ -515,10 +533,23 @@ void *safe_realloc(void *ptr, size_t count)
 		  "Strange, I feel like allocating %d bytes. "
 		  "This must be a bug.", count);
     }
+#ifndef	ORIGINAL
+    else if(count == 0) {
+	/* To be safe: don't ever try to allocate 0 bytes
+	 * Different malloc's behave differently.
+	 */
+	return safe_realloc(ptr, 1);
+    }
+#endif	/* ORIGINAL */
     else if((p = (void *)realloc(ptr, count)) != NULL)
 	return p;
     else
     {
+#ifdef	ORIGINAL
+	if(count == 0)
+	    return safe_malloc(1);
+	    /* RAKK/HIOENS: Hmm, should that have been safe_realloc(ptr,1) ? */
+#endif	/* ORIGINAL */
 	errflag = 1;
 	ctl->cmsg(CMSG_FATAL, VERB_NORMAL,
 		  "Sorry. Couldn't realloc %d bytes.", count);
