@@ -1,6 +1,6 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -169,6 +169,8 @@ void readmidi_add_event(MidiEvent *a_event)
     at = a_event->time;
     newev = alloc_midi_event();
     newev->event = *a_event;	/* assign by value!!! */
+    if(at < 0)	/* for safety */
+	at = newev->event.time = 0;
 
     if(at >= current_midi_point->event.time)
     {
@@ -540,6 +542,8 @@ int parse_sysex_event_multi(uint8 *val, int32 len, MidiEvent *ev, int32 at)
 	current_file_info->mid = val[0];
 
     /* XG Multi Part Data parameter change */
+    /* There are two ways to do this, neither of which match the XG spec... */
+    /* All variables given in a single big block */
     if(len >= 10 &&
        val[0] == 0x43 && /* Yamaha ID */
        val[2] == 0x4C && /* XG Model ID */
@@ -605,32 +609,60 @@ int parse_sysex_event_multi(uint8 *val, int32 len, MidiEvent *ev, int32 at)
 	    num_events++;
 	}
     }
-
-    /* GS program change */
-    /* I could not find any documentation on this, so I reverse engineered
-       which bytes do what from midi I have.  It appears to work correctly. */
-    if(len == 11 &&
-       val[0] == 0x41 && /* Roland ID */
-       val[1] == 0x10 && /* Device ID */
-       val[2] == 0x42 && /* GS Model ID */
-       val[3] == 0x12 && /* Data Set Command */
-       val[4] == 0x40 && /* I wish I knew what this was... */
-       (val[5] >= 0x10 && val[5] < 0x20))   /* channel command */
+    /* Or you can specify them one SYSEX event at a time... */
+    else if(len == 8 &&
+       val[0] == 0x43 && /* Yamaha ID */
+       val[2] == 0x4C && /* XG Model ID */
+       val[3] == 0x08)   /* Multi Part Data parameter change */
     {
 	uint8 p;				/* Channel part number [0..15] */
+	int ent;				/* Entry # of sub-event */
 
-	p = val[5] & 0x0F;
-	if(p == 0)
-	    p = 9;
-	else if(p <= 9)
-	    p--;
-	p = MERGE_CHANNEL_PORT(p);
+	p = val[4];
+	ent = val[5];
 
-	MIDIEVENT(at, ME_TONE_BANK_MSB, p, val[7], 0);
-	MIDIEVENT(at, ME_TONE_BANK_LSB, p, 1, 0);     /* assume SC-55 ??? */
-	MIDIEVENT(at, ME_PROGRAM, p, val[8], 0);
+	switch(ent) {
 
-	num_events = 2;
+		case 0x01:	/* bank select MSB */
+		  MIDIEVENT(at, ME_TONE_BANK_MSB, p, val[6], 0);
+		  break;
+
+		case 0x02:	/* bank select LSB */
+		  MIDIEVENT(at, ME_TONE_BANK_LSB, p, val[6], 0);
+		  break;
+
+		case 0x03:	/* program number */
+		  MIDIEVENT(at, ME_PROGRAM, p, val[6], 0);
+		  break;
+
+		case 0x08:	/* note shift ? */
+		  MIDIEVENT(at, ME_KEYSHIFT, p, val[6], 0);
+
+		case 0x0B:	/* volume */
+		  MIDIEVENT(at, ME_MAINVOLUME, p, val[6], 0);
+		  break;
+
+		case 0x0E:	/* pan */
+		  if(val[6] == 0) {
+			MIDIEVENT(at, ME_RANDOM_PAN, p, 0, 0);
+		  }
+		  else {
+			MIDIEVENT(at, ME_PAN, p, val[6], 0);
+		  }
+		  break;
+
+		case 0x12:	/* chorus send */
+		  MIDIEVENT(at, ME_CHORUS_EFFECT, p, val[6], 0);
+		  break;
+
+		case 0x13:	/* reverb send */
+		  MIDIEVENT(at, ME_REVERB_EFFECT, p, val[6], 0);
+		  break;
+
+		default:
+		  break;
+	}
+	num_events++;
     }
 
     /* GS bank+program change */
@@ -1088,7 +1120,7 @@ int dump_current_timesig(MidiEvent *codes, int maxlen)
 /* Read a SMF track */
 static int read_smf_track(struct timidity_file *tf, int trackno, int rewindp)
 {
-    int32 len;
+    int32 len, next_pos, pos;
     char tmp[4];
     int lastchan, laststatus;
     int me, type, a, b, c;
@@ -1105,6 +1137,7 @@ static int read_smf_track(struct timidity_file *tf, int trackno, int rewindp)
 	return -1;
     }
     len = BE_LONG(len);
+    next_pos = tf_tell(tf) + len;
     if(strncmp(tmp, "MTrk", 4))
     {
 	ctl->cmsg(CMSG_ERROR, VERB_NORMAL,
@@ -1286,6 +1319,9 @@ static int read_smf_track(struct timidity_file *tf, int trackno, int rewindp)
 		    break;
 
 		  case 0x2F: /* End of Track */
+		    pos = tf_tell(tf);
+		    if(pos < next_pos)
+			tf_seek(tf, next_pos - pos, SEEK_CUR);
 		    return 0;
 
 		  case 0x51: /* Tempo */
@@ -1809,10 +1845,17 @@ static MidiEvent *groom_list(int32 divisions, int32 *eventsp, int32 *samplesp)
 		switch(bank_msb[ch])
 		{
 		  case 0: /* Normal */
-		    ctl->cmsg(CMSG_INFO, VERB_DEBUG, "(XG ch=%d Normal voice)",
-			      ch);
-		    midi_drumpart_change(ch, 0);
-		    mapID[ch] = XG_NORMAL_MAP;
+		    if(ch == 9  && bank_lsb[ch] == 127 && mapID[ch] == XG_DRUM_MAP) {
+		      /* FIXME: Why this part is drum?  Is this correct? */
+		      ctl->cmsg(CMSG_WARNING, VERB_NORMAL,
+				"Warning: XG bank 0/127 is found. It may be not correctly played.");
+		      ;
+		    } else {
+		      ctl->cmsg(CMSG_INFO, VERB_DEBUG, "(XG ch=%d Normal voice)",
+				ch);
+		      midi_drumpart_change(ch, 0);
+		      mapID[ch] = XG_NORMAL_MAP;
+		    }
 		    break;
 		  case 64: /* SFX voice */
 		    ctl->cmsg(CMSG_INFO, VERB_DEBUG, "(XG ch=%d SFX voice)",
@@ -2757,6 +2800,8 @@ char *get_midi_title(char *filename)
 
     for(trk = 0; trk < tracks; trk++)
     {
+	int32 next_pos, pos;
+
 	if(trk >= 1 && karaoke_format == -1)
 	    break;
 
@@ -2766,6 +2811,7 @@ char *get_midi_title(char *filename)
 	if(memcmp(tmp, "MTrk", 4))
 	    break;
 
+	next_pos = tf_tell(tf) + len;
 	laststatus = -1;
 	for(;;)
 	{
@@ -2842,7 +2888,12 @@ char *get_midi_title(char *filename)
 		    reuse_mblock(&tmpbuffer);
 		}
 		else if(type == 0x2F)
+		{
+		    pos = tf_tell(tf);
+		    if(pos < next_pos)
+			tf_seek(tf, next_pos - pos, SEEK_CUR);
 		    break; /* End of track */
+		}
 		else
 		    skip(tf, len);
 	    }
