@@ -1,6 +1,6 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -56,17 +56,17 @@
                 v0 = (int32)src[(ofs>>FRACTION_BITS)-1]; \
                 v3 = (int32)src[(ofs>>FRACTION_BITS)+2]; \
                 ofs &= FRACTION_MASK; \
-                temp=v2; \
-		v2 = (6*v2 + \
-		      (((((((5*v3 - 11*v2 + 7*v1 - v0)>>1)* \
-		       ofs)>>FRACTION_BITS)*ofs)>>(FRACTION_BITS+1))-1))*ofs; \
-                ofs = (1L << FRACTION_BITS) - ofs; \
-		v1 = (6*v1 + \
-		      (((((((5*v0 - 11*v1 + 7*temp - v3)>>1)* \
-		       ofs)>>FRACTION_BITS)*ofs)>>(FRACTION_BITS+1))-1))*ofs; \
-		v1 = (v1 + v2)/(6L<<FRACTION_BITS); \
-		*dest++ = (v1 > 32767)? 32767: ((v1 < -32768)? -32768: v1); \
-		ofs=ofsd; \
+	        temp=v2; \
+ 		v2 = (6*v2+((((5*v3 - 11*v2 + 7*v1 - v0)>>2)* \
+ 		     (ofs+(1L<<FRACTION_BITS))>>FRACTION_BITS)* \
+ 		     (ofs-(1L<<FRACTION_BITS))>>FRACTION_BITS)) \
+ 		     *ofs; \
+ 		v1 = (((6*v1+((((5*v0 - 11*v1 + 7*temp - v3)>>2)* \
+ 		     ofs>>FRACTION_BITS)*(ofs-(2L<<FRACTION_BITS)) \
+ 		     >>FRACTION_BITS))*((1L<<FRACTION_BITS)-ofs))+v2) \
+ 		     /(6L<<FRACTION_BITS); \
+ 		*dest++ = (v1 > 32767)? 32767: ((v1 < -32768)? -32768: v1); \
+		ofs = ofsd; \
 	}
 #elif defined(LAGRANGE_INTERPOLATION)
 # define INTERPVARS      int32   ofsd, v0, v1, v2, v3;
@@ -207,7 +207,7 @@ static sample_t *rs_plain(int v, int32 *countptr)
     {
       FINALINTERP;
       vp->timeout = 1;
-      *countptr-=count+1;
+      *countptr-=count;
     }
 #else /* PRECALC_LOOPS */
     while (count--)
@@ -218,7 +218,7 @@ static sample_t *rs_plain(int v, int32 *countptr)
 	{
 	  FINALINTERP;
 	  vp->timeout = 1;
-	  *countptr-=count+1;
+	  *countptr-=count;
 	  break;
 	}
     }
@@ -546,7 +546,7 @@ static sample_t *rs_vib_plain(int v, int32 *countptr)
 	{
 	  FINALINTERP;
 	  vp->timeout = 1;
-	  *countptr-=count+1;
+	  *countptr-=count;
 	  break;
 	}
     }
@@ -834,7 +834,7 @@ static sample_t *porta_resample_voice(int v, int32 *countptr, int mode)
 	resampler(v, &i, mode);
 	resample_buffer_offset += i;
 
-	if(!loop && vp->status == VOICE_FREE)
+	if(!loop && (i == 0 || vp->status == VOICE_FREE))
 	    break;
 	cc -= i;
     }
@@ -871,7 +871,9 @@ sample_t *resample_voice(int v, int32 *countptr)
     Voice *vp=&voice[v];
     int mode;
 
-    if(!(vp->sample->sample_rate))
+    if(vp->sample->sample_rate == play_mode->rate &&
+       vp->sample->root_freq == freq_table[vp->sample->note_to_use] &&
+       vp->frequency == vp->orig_frequency)
     {
 	int32 ofs;
 
@@ -993,5 +995,11 @@ void pre_resample(Sample * sp)
   sp->loop_end = (int32)(sp->loop_end * a);
   free(sp->data);
   sp->data = (sample_t *) newdata;
+/*
   sp->sample_rate = 0;
+*/
+  sp->root_freq = freq_table[(int) (sp->note_to_use)];
+  sp->sample_rate = play_mode->rate;
+  sp->low_freq = freq_table[0];
+  sp->high_freq = freq_table[127];
 }
