@@ -1,6 +1,6 @@
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -83,8 +83,8 @@ extern char *optarg;
 #include "w32g_utl.h"
 #endif
 
-#define OPTCOMMANDS "4A:aB:b:C:c:D:d:E:eFfg:hI:i:jk:L:M:m:n:O:o:P:p:Q:q:R:rS:s:t:T:UW:w:x:Z:"
-#define INTERACTIVE_INTERFACE_IDS "kmqagrw"
+#define OPTCOMMANDS "4A:aB:b:C:c:D:d:E:eFfg:hI:i:jk:L:M:m:n:O:o:P:p:Q:q:R:rS:s:t:T:UW:w:x:Z:"	/* Only GHJKlNuVvXz are remain... */
+#define INTERACTIVE_INTERFACE_IDS "kmqagrwA"
 
 /* main interfaces (To be used another main) */
 #if defined(main) || defined(ANOTHER_MAIN)
@@ -308,7 +308,7 @@ static void help(void)
   static char *help_args[3];
   FILE *fp;
   static char *help_list[] = {
-" TiMidity++ version %s (C) 1999 Masanao Izumo <mo@goice.co.jp>",
+" TiMidity++ version %s (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>",
 " The original version (C) 1995 Tuukka Toivonen <tt@cgs.fi>",
 " TiMidity is free software and comes with ABSOLUTELY NO WARRANTY.",
 "",
@@ -330,7 +330,7 @@ static void help(void)
 #endif
 "  -A n    Amplify volume by n percent (may cause clipping)",
 "  -a      Enable the antialiasing filter",
-"  -B n    Set number of buffer fragments",
+"  -B n,m  Set number of buffer fragments(n), and buffer size(2^m)",
 "  -C n    Set ratio of sampling and control frequencies",
 "  -c file Read extra configuration file",
 "  -D n    Play drums on channel n",
@@ -504,7 +504,7 @@ NULL
   while(*cmp)
   {
 #ifdef IA_DYNAMIC
-      if((*cmp)->id_character != 0)
+      if((*cmp)->id_character != dynamic_interface_id)
 #endif /* IA_DYNAMIC */
 	  fprintf(fp, "  -i%c     %s" NLS,
 		  (*cmp)->id_character, (*cmp)->id_name);
@@ -521,6 +521,7 @@ NULL
       for(cmp = ctl_list; *cmp; cmp++)
 	  mark[(int)(*cmp)->id_character] = 1;
 
+      if (dynamic_interface_id != 0) mark[(int)dynamic_interface_id] = 0;
       list_dyna_interface(fp, dynamic_lib_root, mark);
   }
 #endif /* IA_DYNAMIC */
@@ -563,7 +564,7 @@ static void interesting_message(void)
   printf(
 NLS
 " TiMidity++ version %s -- MIDI to WAVE converter and player" NLS
-" Copyright (C) 1999 Masanao Izumo <mo@goice.co.jp>" NLS
+" Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>" NLS
 " Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>" NLS
 NLS
 #ifdef __W32__
@@ -777,7 +778,7 @@ int set_ctl(char *cp)
 	    return 0;
 	}
 #ifdef IA_DYNAMIC
-	else if(cmp->id_character == 0) /* Dynamic interface loader */
+	else if(cmp->id_character == dynamic_interface_id) /* Dynamic interface loader */
 	{
 	    if(dynamic_interface_module(*cp) != NULL)
 	    {
@@ -2421,6 +2422,29 @@ static double spectrogram_update_sec = 0.0;
 #endif /* SUPPORT_SOUNDSPEC */
 int opt_buffer_fragments = -1;
 
+static int parse_opt_B(char *opt)
+{
+  char *p;
+  int32 val;
+
+  /* num */
+  if(*opt != ',') {
+    if(set_value(&val, atoi(opt), 0, 1000, "Buffer fragments (num)"))
+      return -1;
+    if(val >= 0)
+      opt_buffer_fragments = val;
+  }
+
+  /* bits */
+  if((p = strchr(opt, ',')) != NULL) {
+    if(set_value(&val, atoi(p + 1), 0, AUDIO_BUFFER_BITS, "Buffer fragments (bit)"))
+      return -1;
+    if(val >= 0)
+      audio_buffer_bits = val;
+  }
+  return 0;
+}
+
 MAIN_INTERFACE int set_tim_opt(int c, char *optarg)
 {
     int32 tmpi32;
@@ -2444,9 +2468,14 @@ MAIN_INTERFACE int set_tim_opt(int c, char *optarg)
 	break;
 
       case 'B':
+	if(parse_opt_B(optarg))
+	  return 1;
+	break;
+#if 0
 	if(set_value(&tmpi32, atoi(optarg), 0, 1000, "Buffer fragments"))
 	    return 1;
 	opt_buffer_fragments = tmpi32;
+#endif
 	break;
 
       case 'b':
@@ -2732,7 +2761,7 @@ static RETSIGTYPE sigterm_exit(int sig)
     s[2] = '\n';
     write(2, s, 3);
 
-    if(sig == SIGINT && intr < 3)
+    if(sig == SIGINT && intr < 0)
     {
 	intr++;
 	signal(SIGINT, sigterm_exit); /* For SysV base */
@@ -3179,6 +3208,12 @@ MAIN_INTERFACE int timidity_play_main(int nfiles, char **files)
     return 0;
 }
 
+#ifdef IA_W32GUI
+int w32gSecondTiMidity(int opt, int argc, char **argv);
+int w32gSecondTiMidityExit(void);
+extern int SecondMode;
+#endif /* IA_W32GUI */
+
 #ifndef __MACOS__
 #ifdef __W32__ /* Windows */
 #if defined(__CYGWIN32__) || defined(__MINGW32__) || !defined(IA_W32GUI)
@@ -3269,6 +3304,13 @@ int main(int argc, char **argv)
     timidity_start_initialize();
 #ifdef IA_W32GUI
     w32g_initialize();
+
+	/* Secondary TiMidity Execute */
+	/*	FirstLoadIniFile(); */
+	if(w32gSecondTiMidity(SecondMode,argc,argv)==FALSE){
+		return 0;
+	}
+ 
     for(c = 1; c < argc; c++)
     {
 	if(is_directory(argv[c]))
@@ -3346,10 +3388,20 @@ int main(int argc, char **argv)
 
     nfiles = argc - optind;
     files  = argv + optind;
-    if(nfiles > 0 && ctl->id_character != 'r')
+    if(nfiles > 0 && ctl->id_character != 'r' && ctl->id_character != 'A')
 	files = expand_file_archives(files, &nfiles);
     if(dumb_error_count)
 	sleep(1);
+
+#ifndef IA_W32GUI
     return timidity_play_main(nfiles, files);
+#else
+	{
+		int res = timidity_play_main(nfiles, files);
+		w32gSecondTiMidityExit();
+		return res;
+	}
+#endif /* IA_W32GUI */
+
 }
 #endif /* __MACOS__ */
