@@ -1,8 +1,6 @@
-#ifndef ___TIMIDITY_H_
-#define ___TIMIDITY_H_
 /*
     TiMidity++ -- MIDI to WAVE converter and player
-    Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+    Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
     Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
     This program is free software; you can redistribute it and/or modify
@@ -20,6 +18,8 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#ifndef ___TIMIDITY_H_
+#define ___TIMIDITY_H_
 
 #if defined(__WIN32__) && !defined(__W32__)
 #define __W32__
@@ -130,10 +130,15 @@ typedef double FLOAT_T;
    You should probably use a larger number for improved performance.
 
 */
+
+#define AUDIO_BUFFER_BITS 12	/* Maxmum audio buffer size (2^bits) */
+
+#ifndef DEFAULT_AUDIO_BUFFER_BITS
 #ifdef __W32__
-#define AUDIO_BUFFER_BITS 12
+#define DEFAULT_AUDIO_BUFFER_BITS 12
 #else
-#define AUDIO_BUFFER_BITS 11
+#define DEFAULT_AUDIO_BUFFER_BITS 11
+#endif
 #endif
 
 
