@@ -1,6 +1,6 @@
 /*
    TiMidity++ -- MIDI to WAVE converter and player
-   Copyright (C) 1999,2000 Masanao Izumo <mo@goice.co.jp>
+   Copyright (C) 1999-2001 Masanao Izumo <mo@goice.co.jp>
    Copyright (C) 1995 Tuukka Toivonen <tt@cgs.fi>
 
    This program is free software; you can redistribute it and/or modify
@@ -42,6 +42,9 @@
 // #define USE_WAVEOUT_EVENT
 
 #if defined(__CYGWIN32__) || defined(__MINGW32__)
+#ifdef HAVE_NEW_MMSYSTEM
+#include <mmsystem.h>
+#else
 /* On cygnus, there is not mmsystem.h for Multimedia API's.
  * mmsystem.h can not distribute becase of Microsoft Lisence
  * Then declare some of them here.
@@ -82,7 +85,7 @@ typedef struct {
     WORD    nBlockAlign;
     WORD    wBitsPerSample;
     WORD    cbSize;
-} WAVEFORMAT;
+} WAVEFORMATEX;
 
 typedef struct waveoutcaps_tag {
     WORD    wMid;
@@ -96,12 +99,12 @@ typedef struct waveoutcaps_tag {
 } WAVEOUTCAPS;
 
 typedef WAVEHDR *LPWAVEHDR;
-typedef WAVEFORMAT *LPWAVEFORMAT;
+typedef WAVEFORMATEX *LPWAVEFORMATEX;
 typedef WAVEOUTCAPS *LPWAVEOUTCAPS;
 typedef UINT MMRESULT;
 
 MMRESULT WINAPI waveOutOpen(LPHWAVEOUT, UINT,
-			    LPWAVEFORMAT, DWORD, DWORD, DWORD);
+			    LPWAVEFORMATEX, DWORD, DWORD, DWORD);
 MMRESULT WINAPI waveOutClose(HWAVEOUT);
 MMRESULT WINAPI waveOutPrepareHeader(HWAVEOUT, LPWAVEHDR, UINT);
 MMRESULT WINAPI waveOutUnprepareHeader(HWAVEOUT, LPWAVEHDR, UINT);
@@ -113,19 +116,9 @@ MMRESULT WINAPI waveOutGetDevCaps(UINT, LPWAVEOUTCAPS, UINT);
 MMRESULT WINAPI waveOutGetDevCapsA(UINT, LPWAVEOUTCAPS, UINT);
 #define waveOutGetDevCaps waveOutGetDevCapsA
 MMRESULT WINAPI waveOutGetID(HWAVEOUT, UINT*);
+#endif
 #else
 #include <process.h>
-#ifdef HAVE_MMSYSTEM_H
-typedef struct {
-    WORD    wFormatTag;
-    WORD    nChannels;
-    DWORD   nSamplesPerSec;
-    DWORD   nAvgBytesPerSec;
-    WORD    nBlockAlign;
-    WORD    wBitsPerSample;
-    WORD    cbSize;
-} WAVEFORMAT_;
-#endif /* HAVE_MMSYSTEM_H */
 #endif /* __CYGWIN32__ */
 
 #include "timidity.h"
@@ -140,9 +133,10 @@ typedef struct {
 // #define DATA_BLOCK_NUM  (dpm.extra_param[0])
 #define DATA_BLOCK_NUM 16
 #define DATA_MIN_NBLOCKS (DATA_BLOCK_NUM-1)
-volatile int data_block_size = 4*AUDIO_BUFFER_SIZE*4;
-volatile int data_block_time = 400;	// msec
-volatile int data_block_num = 20;
+static int data_block_size;
+static int data_block_time;	// msec
+volatile int data_block_bits = DEFAULT_AUDIO_BUFFER_BITS;
+volatile int data_block_num = 64;
 volatile struct data_block_t
 {
 	HGLOBAL data_hg;
@@ -291,11 +285,7 @@ static void CALLBACK wave_callback(HWAVE hWave, UINT uMsg,
 static int open_output(void)
 {
     int i, j, mono, eight_bit, warnings = 0;
-#ifdef HAVE_MMSYSTEM_H
-    WAVEFORMAT_ wf;
-#else
-    WAVEFORMAT wf;
-#endif /* HAVE_MMSYSTEM_H */
+    WAVEFORMATEX wf;
     WAVEOUTCAPS caps;
     MMRESULT res;
     UINT devid;
@@ -343,22 +333,18 @@ static int open_output(void)
     wf.nAvgBytesPerSec = i;
     wf.nBlockAlign = j;
     wf.wBitsPerSample = eight_bit ? 8 : 16;
-#ifdef HAVE_MMSYSTEM_H
-    wf.cbSize=sizeof(WAVEFORMAT_);
-#else
-    wf.cbSize=sizeof(WAVEFORMAT);
-#endif /* HAVE_MMSYSTEM_H */
+    wf.cbSize=sizeof(WAVEFORMATEX);
 
     dev = 0;
 #ifdef USE_WAVEFORMTHREAD
 	InitWaveformThread();
 #endif
     if (w32_wave_allowsync)
-	res = waveOutOpen (&dev, WAVE_MAPPER, (LPWAVEFORMAT)&wf,
+	res = waveOutOpen (&dev, WAVE_MAPPER, &wf,
 			   (DWORD)wave_callback, 0,
 			   CALLBACK_FUNCTION | WAVE_ALLOWSYNC);
     else
-	res = waveOutOpen (&dev, WAVE_MAPPER, (LPWAVEFORMAT)&wf,
+	res = waveOutOpen (&dev, WAVE_MAPPER, &wf,
 			   (DWORD)wave_callback, 0, CALLBACK_FUNCTION);
     if (res)
     {
@@ -394,11 +380,18 @@ static int open_output(void)
 	      "functionality supported by driver: 0x%x", caps.dwSupport);
 
     /* Prepere audio queue buffer */
-	data_block_size = (int)((double)wf.nAvgBytesPerSec * data_block_time / 1000);
+//    data_block_size = (int)((double)wf.nAvgBytesPerSec * data_block_time / 1000);
+    data_block_size = audio_buffer_size;
+    if(!(dpm.encoding & PE_MONO))
+      data_block_size *= 2;
+    if(dpm.encoding & PE_16BIT)
+      data_block_size *= 2;
+    data_block_time = data_block_size * 1000 / wf.nAvgBytesPerSec;
+
 // PrintfDebugWnd("DATA_BLOCK_SIZE: %ld\n",data_block_size);
     all_data_block = (struct data_block_t *)
-	safe_malloc(data_block_num * sizeof(struct data_block_t));
-	all_data_block_used_flag = (int *)safe_malloc(data_block_num * sizeof(int));
+      safe_malloc(data_block_num * sizeof(struct data_block_t));
+    all_data_block_used_flag = (int *)safe_malloc(data_block_num * sizeof(int));
     for(i = 0; i < data_block_num; i++)
     {
 	struct data_block_t *block;
@@ -526,16 +519,8 @@ static int acntl(int request, void *arg)
 {
     switch(request)
     {
-      case PM_REQ_GETQSIZ:
-#if 0
-		*(int *)arg = data_block_num * AUDIO_BUFFER_SIZE;
-		if(!(dpm.encoding & PE_MONO))
-			*(int *)arg *= 2;
-		if(dpm.encoding & PE_16BIT)
-			*(int *)arg *= 2;
-#else
-		*(int *)arg = data_block_num * data_block_size;
-#endif
+          case PM_REQ_GETQSIZ:
+                *(int *)arg = data_block_num * audio_buffer_size;
 		return 0;
 	  case PM_REQ_DISCARD:
 	  case PM_REQ_FLUSH:
